package org.openxma.dsl.platform.config;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeansException;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.factory.BeanDefinitionStoreException;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.util.StringUtils;

/**
 * 
 * Xproperties based Backport of spring 3.x <code>@Value</code> annotation
 * expression resolving mechanism to spring 2.x. <br/>
 * For the most part this code was copied from
 * stackoverflow.com/questions/317687/inject-property-value-into-spring-bean.
 */
public class XPropertyAnnotationPlaceholderConfigurer extends XPropertiesPlaceholderConfigurer {
	/** Logger available to subclasses */
	protected final Log logger = LogFactory.getLog(getClass());

	@Override
	protected void processProperties(ConfigurableListableBeanFactory beanFactory, Properties properties)
			throws BeansException {
		for (String name : beanFactory.getBeanDefinitionNames()) {
			MutablePropertyValues mpv = beanFactory.getBeanDefinition(name).getPropertyValues();
			Class clazz = beanFactory.getType(name);
			if (processClass(clazz)) {
				if (logger.isDebugEnabled()) {
					logger.debug("Configuring properties for bean=" + name + "[" + clazz + "]");
				}
				for (PropertyDescriptor property : BeanUtils.getPropertyDescriptors(clazz)) {
					Method setter = property.getWriteMethod();
					Method getter = property.getReadMethod();
					XProperty annotation = null;
					if (setter != null && setter.isAnnotationPresent(XProperty.class)) {
						annotation = setter.getAnnotation(XProperty.class);
					} else if (setter != null && getter != null && getter.isAnnotationPresent(XProperty.class)) {
						annotation = getter.getAnnotation(XProperty.class);
					}
					if (annotation != null) {
						String key = getKey(clazz, property.getName(), annotation);
						String value = resolvePlaceholder(key, properties, SYSTEM_PROPERTIES_MODE_FALLBACK);
						if (!StringUtils.hasText(value) && !key.equalsIgnoreCase(clazz.getName() + "." + property.getName())) {
							value = resolvePlaceholder(clazz.getName() + "." + property.getName(), properties);
						}
						if (!StringUtils.hasText(value)) {
							value = annotation.defaultValue();
						}
						if (!StringUtils.hasText(value)) {
							throw new BeanDefinitionStoreException("No such property=[" + annotation.key()
									+ "] found in properties.");
						}
						if (logger.isDebugEnabled())
							logger.debug("Setting property=[" + clazz.getName() + "." + property.getName() + "] value=["
									+ annotation.key() + "=" + value + "]");
						mpv.addPropertyValue(property.getName(), value);
					}
				}
				for (Field field : clazz.getDeclaredFields()) {
					if (logger.isDebugEnabled()) {
						logger.debug("Examining field=[" + clazz.getName() + "." + field.getName() + "]");
					}
					if (field.isAnnotationPresent(XProperty.class)) {
						XProperty annotation = field.getAnnotation(XProperty.class);
						PropertyDescriptor property = BeanUtils.getPropertyDescriptor(clazz, field.getName());
						if (property.getWriteMethod() == null) {
							throw new BeanDefinitionStoreException("Setter for property=[" + clazz.getName() + "."
									+ field.getName() + "] not available.");
						}
						String key = getKey(clazz, field.getName(), annotation);
						Object value = resolvePlaceholder(key, properties, SYSTEM_PROPERTIES_MODE_FALLBACK);
						if (value == null) {
							value = annotation.defaultValue();
						}
						if (value == null) {
							throw new BeanDefinitionStoreException("No such property=[" + annotation.key()
									+ "] found in properties.");
						}
						if (logger.isDebugEnabled()) {
							logger.debug("Setting property=[" + clazz.getName() + "." + field.getName() + "] value=["
									+ annotation.key() + "=" + value + "]");
						}
						mpv.addPropertyValue(property.getName(), value);
					}
				}
			}
		}
	}

	private String getKey(Class clazz, String field, XProperty annotation) {
		return StringUtils.hasText(annotation.key()) ? annotation.key() : clazz.getName() + "." + field;
	}

	private boolean processClass(Class clazz) {
		return clazz != null && !clazz.getName().startsWith("org.springframework");
	}
}
