package org.openxma.dsl.platform.converter.impl;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.beanutils.converters.BigDecimalConverter;
import org.apache.commons.beanutils.converters.BooleanConverter;
import org.apache.commons.beanutils.converters.DoubleConverter;
import org.apache.commons.beanutils.converters.FloatConverter;
import org.apache.commons.beanutils.converters.IntegerConverter;
import org.apache.commons.beanutils.converters.LongConverter;
import org.apache.commons.beanutils.converters.StringConverter;
import org.openxma.dsl.platform.converter.Converter;
import org.openxma.dsl.platform.service.MapperConfig;
import org.openxma.dsl.platform.service.MapperContextHolder;
import org.springframework.stereotype.Service;

@Service
public class DataTypeConverter implements Converter {
    public static final String DEFAULT_DATE_FORMAT = "DEFAULT_DATE_FORMAT";
    private static final String DEFAULT_TIMESTAMP_FORMAT = "DEFAULT_TIMESTAMP_FORMAT";

    private static final Map<String, DateFormat> dateFormatPatternCache = new ConcurrentHashMap<String, DateFormat>(10,
            10, 10);
    private static final Map<String, DateFormat> timestampFormatPatternCache = new ConcurrentHashMap<String, DateFormat>(
            10, 10, 10);
    private static final org.apache.commons.beanutils.Converter booleanConverter = new BooleanConverter();

    private static final org.apache.commons.beanutils.Converter longConverter = new LongConverter();

    private static final org.apache.commons.beanutils.Converter doubleConverter = new DoubleConverter();

    private static final org.apache.commons.beanutils.Converter floatConverter = new FloatConverter();

    private static final org.apache.commons.beanutils.Converter bigDecimalConverter = new BigDecimalConverter();

    private static final org.apache.commons.beanutils.Converter integerConverter = new IntegerConverter();

    private static final org.apache.commons.beanutils.Converter stringConverter = new StringConverter();

    private String defaultDateFormatPattern;

    static {
        dateFormatPatternCache.put(DEFAULT_DATE_FORMAT, new SimpleDateFormat());
        timestampFormatPatternCache.put(DEFAULT_TIMESTAMP_FORMAT, new SimpleDateFormat("yyyy-mm-dd hh:mm:ss"));
    }

    public void setDefaultDateFormatPattern(String defaultDateFormatPattern) {
        this.defaultDateFormatPattern = defaultDateFormatPattern;
        dateFormatPatternCache.put(DEFAULT_DATE_FORMAT, new SimpleDateFormat(this.defaultDateFormatPattern));
    }

    public void resetDefaultDateFormatPattern() {
        dateFormatPatternCache.put(DEFAULT_DATE_FORMAT, new SimpleDateFormat());
    }

    public DateFormat getCachedDateFormat(String pattern) {
        return dateFormatPatternCache.get(pattern);
    }

    public int getDateFormatPatternCacheSize() {
        return dateFormatPatternCache.size();
    }

    @SuppressWarnings("unchecked")
    public <T> T convert(Class<T> destClass, Object sourceObject) {
        if (sourceObject instanceof Integer) {
            return convertInteger(destClass, sourceObject);
        } else if (sourceObject instanceof String) {
            return convertString(destClass, sourceObject);
        } else if (sourceObject instanceof Timestamp) {
            return convertTimestamp(destClass, sourceObject);
        } else if (sourceObject instanceof Date) {
            return convertDate(destClass, sourceObject);
        } else if (sourceObject instanceof BigDecimal) {
            return convertBigDecimal(destClass, sourceObject);
        } else if (sourceObject instanceof Boolean) {
            return convertBoolean(destClass, sourceObject);
        } else if (sourceObject instanceof Double) {
            return convertDouble(destClass, sourceObject);
        } else if (sourceObject instanceof Float) {
            return convertFloat(destClass, sourceObject);
        } else if (sourceObject instanceof Long) {
            return convertLong(destClass, sourceObject);
        }
        return null;
    }

    @SuppressWarnings("unchecked")
    private <T> T convertInteger(Class<T> destClass, Object sourceObject) {
        return (T) integerConverter.convert(destClass, sourceObject);
    }

    @SuppressWarnings("unchecked")
    private <T> T convertString(Class<T> destClass, Object sourceObject) {
        if (destClass == java.util.Date.class)
            return (T) new DateConverter(getCurrentDateFormat()).convert(destClass, sourceObject);
        else if (destClass == java.sql.Timestamp.class)
            return (T) new DateConverter(getCurrentTimestampFormat()).convert(destClass, sourceObject);
        else if (destClass == java.lang.Boolean.class)
            return (T) booleanConverter.convert(destClass, sourceObject);
        else if (destClass == BigDecimal.class)
            return (T) bigDecimalConverter.convert(destClass, sourceObject);
        else if (destClass == Double.class)
            return (T) doubleConverter.convert(destClass, sourceObject);
        else if (destClass == Float.class)
            return (T) floatConverter.convert(destClass, sourceObject);
        else if (destClass == Integer.class)
            return (T) integerConverter.convert(destClass, sourceObject);
        else if (destClass == Long.class)
            return (T) longConverter.convert(destClass, sourceObject);
        else
            return (T) stringConverter.convert(destClass, sourceObject);
    }

    @SuppressWarnings("unchecked")
    private <T> T convertTimestamp(Class<T> destClass, Object sourceObject) {
        if (destClass == String.class) {
            // instantiate a new instance, because it holds state, the date format
            org.apache.commons.beanutils.converters.DateConverter converter = new org.apache.commons.beanutils.converters.DateConverter();
            String timestampFormatPattern = getCurrentTimestampFormatPattern();
            if (null != timestampFormatPattern)
                converter.setPattern(timestampFormatPattern);
            return (T) converter.convert(destClass, sourceObject);
        } else {
            return (T) new DateConverter(getCurrentTimestampFormat()).convert(destClass, sourceObject);
        }
    }

    @SuppressWarnings("unchecked")
    private <T> T convertDate(Class<T> destClass, Object sourceObject) {
        if (destClass == String.class) {
            // instantiate a new instance, because it holds state, the date format
            org.apache.commons.beanutils.converters.DateConverter converter = new org.apache.commons.beanutils.converters.DateConverter();
            String dateFormatPattern = getCurrentDateFormatPattern();
            if (null != dateFormatPattern)
                converter.setPattern(dateFormatPattern);
            return (T) converter.convert(destClass, sourceObject);
        } else {
            return (T) new DateConverter(getCurrentDateFormat()).convert(destClass, sourceObject);
        }
    }

    @SuppressWarnings("unchecked")
    private <T> T convertBigDecimal(Class<T> destClass, Object sourceObject) {
        if (destClass == java.util.Date.class)
            // instantiate a new instance, because it holds state, the date format
            return (T) new DateConverter(getCurrentDateFormat()).convert(destClass, sourceObject);
        else if (destClass == java.sql.Timestamp.class)
            // instantiate a new instance, because it holds state, the date format
            return (T) new DateConverter(getCurrentTimestampFormat()).convert(destClass, sourceObject);
        else
            return (T) bigDecimalConverter.convert(destClass, sourceObject);
    }

    @SuppressWarnings("unchecked")
    private <T> T convertBoolean(Class<T> destClass, Object sourceObject) {
        return (T) booleanConverter.convert(destClass, sourceObject);
    }

    @SuppressWarnings("unchecked")
    private <T> T convertDouble(Class<T> destClass, Object sourceObject) {
        return (T) doubleConverter.convert(destClass, sourceObject);
    }

    @SuppressWarnings("unchecked")
    private <T> T convertFloat(Class<T> destClass, Object sourceObject) {
        return (T) floatConverter.convert(destClass, sourceObject);
    }

    @SuppressWarnings("unchecked")
    private <T> T convertLong(Class<T> destClass, Object sourceObject) {
        if (destClass == java.util.Date.class || destClass == java.sql.Timestamp.class)
            return (T) new DateConverter(getCurrentDateFormat()).convert(destClass, sourceObject);
        else if (destClass == java.sql.Timestamp.class)
            return (T) new DateConverter(getCurrentTimestampFormat()).convert(destClass, sourceObject);
        else
            return (T) longConverter.convert(destClass, sourceObject);
    }

    private DateFormat getCurrentDateFormat() {
        DateFormat currentDateFormat = refreshDateFormatPatternCache();
        return null == currentDateFormat ? dateFormatPatternCache.get(DEFAULT_DATE_FORMAT) : currentDateFormat;
    }

    private DateFormat getCurrentTimestampFormat() {
        DateFormat currentTimestampFormat = refreshTimestampFormatPatternCache();
        return null == currentTimestampFormat ? timestampFormatPatternCache.get(DEFAULT_TIMESTAMP_FORMAT)
                : currentTimestampFormat;
    }

    private String getCurrentTimestampFormatPattern() {
        return null == refreshTimestampFormatPatternCache() ? null : MapperContextHolder.getMapperConfig()
                .getTimestampPattern();
    }

    private String getCurrentDateFormatPattern() {
        return null == refreshDateFormatPatternCache() ? null : MapperContextHolder.getMapperConfig().getDatePattern();
    }

    private DateFormat refreshTimestampFormatPatternCache() {
        MapperConfig mapperConfig = MapperContextHolder.getMapperConfig();
        if (null == mapperConfig || null == mapperConfig.getTimestampPattern()
                || mapperConfig.getTimestampPattern().trim().length() == 0)
            return null;

        if (!timestampFormatPatternCache.containsKey(mapperConfig.getTimestampPattern()))
            timestampFormatPatternCache.put(mapperConfig.getTimestampPattern(),
                    new SimpleDateFormat(mapperConfig.getTimestampPattern()));

        return timestampFormatPatternCache.get(mapperConfig.getTimestampPattern());
    }

    private DateFormat refreshDateFormatPatternCache() {
        MapperConfig mapperConfig = MapperContextHolder.getMapperConfig();
        if (null == mapperConfig || null == mapperConfig.getDatePattern()
                || mapperConfig.getDatePattern().trim().length() == 0)
            return null;

        if (!dateFormatPatternCache.containsKey(mapperConfig.getDatePattern()))
            dateFormatPatternCache.put(mapperConfig.getDatePattern(),
                    new SimpleDateFormat(mapperConfig.getDatePattern()));

        return dateFormatPatternCache.get(mapperConfig.getDatePattern());
    }

}
